
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_LOG_H_
#define _MOOF_LOG_H_

/**
 * @file log.h
 * Functions related to logging the process.
 * The logging functions are logError(), logWarning(), logInfo(), and
 * logDebug(), listed from most critical to least critical.
 */

#include <cstdlib>		// exit
#include <iostream>


/**
 * Macro which tests an assertion and issues an logError() and exits if false.
 * @param X test to perform
 */

#undef ASSERT

#if NDEBUG
#define ASSERT(X)
#else
#define ASSERT(X) if (!(X)) Mf::logError << \
	"false assertion at " << __FILE__ << ":" << __LINE__ << ", " << #X, exit(1)
#endif


namespace Mf {


class Log
{
public:

	enum Level
	{
		NONE		= 0,		///< Disable all logging.
		ERRORR		= 1,		///< Log only errors.
		WARNING		= 2,		///< Log warnings and errors.
		INFO		= 3,		///< Log everything.
	};

	static void setLevel(Level level);
	static Level getLevel();


	Log(Level level, const char* type) :
		mLevel(level),
		mType(type) /* only pass literal strings */ {}


	template <typename T>
	void operator () (const T& item)
	{
		*this << item << std::endl;
	}

private:

	template <typename T> friend std::ostream& operator << (Log&, const T&);

	static Level	gLevel;

	Level			mLevel;
	const char*		mType;
};


extern std::ostream&	log;
extern std::ostream&	nullLog;

extern Log logError;
extern Log logWarning;
extern Log logInfo;


template <typename T>
inline std::ostream& operator << (Log& logObj, const T& item)
{
	if (Log::gLevel < logObj.mLevel) return nullLog;
	return log << logObj.mType << item;
}


class Script;
void importLogFunctions(Script& script);


} // namespace Mf

#endif // _MOOF_LOG_H_

