
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_RAY_HH_
#define _MOOF_RAY_HH_

#include <Moof/Drawable.hh>
#include <Moof/Math.hh>
#include <Moof/OpenGL.hh>
#include <Moof/Texture.hh>


namespace Mf {


/**
 * A line that goes to infinity.
 */

template <int D>
struct Ray : public Drawable
{
	typedef cml::vector< Scalar, cml::fixed<D> >	Vector;

	// solution = point + t*direction
	Vector	point;
	Vector	direction;

	struct Intersection
	{
		Scalar	distance;	// distance from the origin to the nearest point
		Vector	normal;		// surface normal at intersection point

		bool operator < (const Intersection& rhs)
		{
			return distance < rhs.distance;
		}
	};

	void solve(Vector& p, Scalar t) const
	{
		p = point + t*direction;
	}

	void draw(Scalar alpha = 0.0) const
	{
		Vector end = point + 1000.0 * direction;
		// TODO this is kinda cheesy

		Mf::Texture::resetBind();
		glBegin(GL_LINES);
			glVertex(point);
			glVertex(end);
		glEnd();
	}

	void normalize()
	{
		direction.normalize();
	}
};


} // namespace Mf

#endif // _MOOF_RAY_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

