
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_SHAPE_HH_
#define _MOOF_SHAPE_HH_

#include <Moof/Drawable.hh>
#include <Moof/Math.hh>
#include <Moof/OpenGL.hh>
#include <Moof/Ray.hh>


// Frustum
// Plane (can construct from Triangle<3>)
// Ray
// Shape<>
// +- Line<>
// +- Ball<>
// |  Circle		<- Ball<2>
// |  Sphere		<- Ball<3>
// +- Box<>
// |  Rectangle		<- Box<2>
// |  Aabb			<- Box<3>
// +- Polygon<>
// |  Triangle		<- Polygon<3>
// +- Cylinder
// +- Cone


namespace Mf {


template <int D>
class Shape
{
public:

	virtual ~Shape() {}

	/**
	 * Checks if this shape is intersected by a given ray.  If so, returns the
	 * distance from the start of the ray to the shape and information about the
	 * intersection via the 2nd parameter.  A negative value is returned if
	 * there is no intersection.
	 */
	virtual bool intersectRay(const Ray<D>& ray,
			typename Ray<D>::Intersection& hit)
	{
		return SCALAR(-1.0);
	}
};


} // namespace Mf

#endif // _MOOF_SHAPE_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

